#!/bin/bash

set -e
set -o pipefail

readonly IFS=$'\n\t'
readonly REDIS_SERVER_PATH=$(which redis-server)
readonly REDIS_CLI_PATH=$(which redis-cli)
readonly NODE_PORT="6379"

if [ -z $NODE_IP_ADDR ]; then
	echo "Redis cluster needs the Node IP address"
	exit 1
fi

start_cleanup_daemon() {
	echo "Starting cleanup daemon"
	exec env REDIS_CLI_PATH=$REDIS_CLI_PATH \
		NODE_IP_ADDR=$NODE_IP_ADDR \
		NODE_PORT=$NODE_PORT \
		/bin/bash /usr/local/bin/cleanup.sh
}

create_cluster() {
	echo "------------------------------------------------"
	echo "------------------------------------------------"
	echo "Creating a cluster on: $NODE_IP_ADDR"
	echo "------------------------------------------------"
	echo "------------------------------------------------"
	# waiting for all nodes to boot up
	sleep 10
	$REDIS_CLI_PATH --cluster create \
		"$NODE_IP_ADDR:6379" \
		"$NODE_IP_ADDR:6380" \
		"$NODE_IP_ADDR:6381" \
		--cluster-yes
}

join_existing_cluster() {
	echo "------------------------------------------------"
	echo "------------------------------------------------"
	echo "Connecting to existing cluster on: $JOIN_IP_ADDR"
	echo "------------------------------------------------"
	echo "------------------------------------------------"

	# waiting for all nodes to boot up
	sleep 10
	$REDIS_CLI_PATH --cluster \
		add-node $NODE_IP_ADDR:6379 $JOIN_IP_ADDR:6379

	sleep 2
	$REDIS_CLI_PATH --cluster \
		add-node $NODE_IP_ADDR:6380 $JOIN_IP_ADDR:6380

	sleep 2
	$REDIS_CLI_PATH --cluster \
		add-node $NODE_IP_ADDR:6381 $JOIN_IP_ADDR:6381
}

start_redis_servers() {
	echo "Starting redis server"
	local node_0_conf=/opt/jfrog/redis/node_0/redis.conf
	local node_1_conf=/opt/jfrog/redis/node_1/redis.conf
	local node_2_conf=/opt/jfrog/redis/node_2/redis.conf

	pushd /opt/jfrog/redis/node_0
	$REDIS_SERVER_PATH $node_0_conf &
	popd

	pushd /opt/jfrog/redis/node_1
	$REDIS_SERVER_PATH $node_1_conf &
	popd

	pushd /opt/jfrog/redis/node_2
	$REDIS_SERVER_PATH $node_2_conf &
	popd
}

main() {
  echo "Starting redis cluster"
	start_redis_servers
	if [ ! -z "$JOIN_IP_ADDR" ]; then
		join_existing_cluster
	else
		create_cluster
	fi
	start_cleanup_daemon
}

main
